/* Copyright 1998-2011 Research Foundation State University of New York */

/* This file is part of QuB.                                            */

/* QuB is free software; you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* QuB is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License,   */
/* named LICENSE.txt, in the QuB program directory.  If not, see        */
/* <http://www.gnu.org/licenses/>.                                      */

#ifdef _USRDLL
  #define _QTR_PY_
#endif
#include "qubtree_NodeFunc.h"


QTR_NodePyFunc::QTR_NodePyFunc( PyObject *fn )
  : func( fn )
{
	Py_INCREF( fn );
}

QTR_NodePyFunc::~QTR_NodePyFunc()
{
	Py_DECREF( func );
}

QUB_Tree QTR_NodePyFunc::NodeCall( QUB_Tree arg )
{
	QUB_Tree result;
	if ( PyCallable_Check( func ) ) {
		PyObject *tup = Py_BuildValue( "(N)", qubtree_node_using( arg.getImpl() ) );
		PyObject *pyres = PyEval_CallObject( func, tup );
		Py_DECREF( tup );

		if ( pyres == NULL ) {
			PyErr_Clear();
		}
		else {
			if ( qubtree_node_check( pyres ) )
				result = qubtree_node_obj( pyres );
			Py_DECREF( pyres );
		}
	}
	return result;
}

QTR_NodeNodePyFunc::QTR_NodeNodePyFunc( PyObject *fn )
  : func( fn )
{
  Py_INCREF( fn );
}

QTR_NodeNodePyFunc::~QTR_NodeNodePyFunc()
{
  Py_DECREF( func );
}

QUB_Tree QTR_NodeNodePyFunc::NodeNodeCall( QUB_Tree arg1, QUB_Tree arg2 )
{
	QUB_Tree result;
	if ( PyCallable_Check( func ) ) {
		PyObject *tup = Py_BuildValue( "(NN)", qubtree_node_using( arg1.getImpl() ),
											   qubtree_node_using( arg2.getImpl() ) );
		PyObject *pyres = PyEval_CallObject( func, tup );
		Py_DECREF( tup );

		if ( pyres == NULL ) {
			PyErr_Clear();
		}
		else {
			if ( qubtree_node_check( pyres ) )
				result = qubtree_node_obj( pyres );
			Py_DECREF( pyres );
		}
	}
	return result;
}

